﻿-- ********************************************************************************
-- Data Broker Volume Control (Broker_Volume)
-- A volume control for Data Broker.
-- By: Shenton
--
-- Core.lua
-- ********************************************************************************

-- Ace libs (<3)
Broker_Volume = LibStub("AceAddon-3.0"):NewAddon("Broker_Volume", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("Broker_Volume", false);

-- LibDBIcon
Broker_Volume.icon = LibStub("LibDBIcon-1.0");

-- ********************************************************************************
-- Variables
-- ********************************************************************************

-- AddOn version
Broker_Volume.version = GetAddOnMetadata("Broker_Volume", "Version");

-- Text colors
Broker_Volume.color = {};
Broker_Volume.color["RED"] = "|cFFFF3333";
Broker_Volume.color["GREEN"] = "|cFF33FF99";
Broker_Volume.color["WHITE"] = "|cFFFFFFFF";
Broker_Volume.color["GRAY"] = "|cFF999999";
Broker_Volume.color["WARRIOR"] = "|cFFC79C6E";
Broker_Volume.color["RESET"] = "|r";

-- ********************************************************************************
-- Dropdown Menu
-- ********************************************************************************

--- The dropdown menu structure function
-- @usage Called by ToggleDropDownMenu()
-- @param self Frame object
-- @param level Nesting level
local function DropdownMenu(self, level)
	if ( not level ) then return; end

	local info = self.info;
    local db = Broker_Volume.config.profile;
    local master = Broker_Volume:GetVolumePercent("Sound_MasterVolume");
    local effects = Broker_Volume:GetVolumePercent("Sound_SFXVolume");
    local music = Broker_Volume:GetVolumePercent("Sound_MusicVolume");
    local ambience = Broker_Volume:GetVolumePercent("Sound_AmbienceVolume");

	wipe(info);

	if ( level == 1 ) then
		-- Menu title
		info.isTitle = true;
		info.text = L["ADDON_NAME"];
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.keepShownOnClick = true;
		info.hasArrow = true;
		info.disabled = nil;
		info.isTitle = nil;

        -- Master
        info.text = "   "..L["MASTER_VOLUME"];
        info.value = "MASTER";
        UIDropDownMenu_AddButton(info, level);

        -- Effects
        info.text = "   "..L["EFFECTS_VOLUME"];
        info.value = "EFFECTS";
        UIDropDownMenu_AddButton(info, level);

        -- Music
        info.text = "   "..L["MUSIC_VOLUME"];
        info.value = "MUSIC";
        UIDropDownMenu_AddButton(info, level);

        -- Ambience
        info.text = "   "..L["AMBIENCE_VOLUME"];
        info.value = "AMBIENCE";
        UIDropDownMenu_AddButton(info, level);

		-- Blank separator
		wipe(info);
		info.disabled = true;
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.disabled = nil;

		-- Options
		info.text = "   "..L["OPTIONS"];
		info.value = "OPTIONS";
		info.hasArrow = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.keepShownOnClick = nil;
		info.hasArrow = nil;

		-- Close
		info.text = L["CLOSE"];
		info.func = function() CloseDropDownMenus(); end;
		UIDropDownMenu_AddButton(info, level);
	elseif ( level == 2 ) then
		if ( UIDROPDOWNMENU_MENU_VALUE == "MASTER" ) then -- Master
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == master ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_MasterVolume", Broker_Volume:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "EFFECTS" ) then -- Effects
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == effects ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_SFXVolume", Broker_Volume:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "MUSIC" ) then -- Music
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == music ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_MusicVolume", Broker_Volume:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
        elseif ( UIDROPDOWNMENU_MENU_VALUE == "AMBIENCE" ) then -- Ambience
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == ambience ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_AmbienceVolume", Broker_Volume:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "OPTIONS" ) then -- Options
			wipe(info);

			-- Set options
			info.keepShownOnClick = true;

			-- Display spec name
			info.text = L["BLIZLIKE_MUTE"];
			info.checked = db.blizlike;
			info.func = function() db.blizlike = not db.blizlike; end;
			UIDropDownMenu_AddButton(info, level);

			-- Show/hide minimap icon
			info.text = L["SHOW_HIDE_MINIMAP"];
			info.checked = not db.minimap.hide;
			info.func = function()
				db.minimap.hide = not db.minimap.hide;
				Broker_Volume:ShowHideMinimap();
			end;
			UIDropDownMenu_AddButton(info, level);
		end
	end
end

-- ********************************************************************************
-- Functions
-- ********************************************************************************

--- Send a message to the chat frame with the addon name colored
-- @param text The message to display
-- @param color Bool, if true will color in red
function Broker_Volume:Message(text, color)
	if ( color ) then
		color = Broker_Volume.color["RED"];
	else
		color = Broker_Volume.color["GREEN"]
	end

	DEFAULT_CHAT_FRAME:AddMessage(color..L["ADDON_NAME"]..": "..Broker_Volume.color["RESET"]..text);
end

--- Handle the slash command
-- @param input The string returned after the command
function Broker_Volume:SlashCommand(input)
	local arg1, arg2 = string.match(input, "(%a*)%s?(.*)");

	if ( arg1 == "" ) then
		Broker_Volume:Message(L["COMMAND_USAGE_1"]);
		Broker_Volume:Message(L["COMMAND_USAGE_2"]);
		Broker_Volume:Message(L["COMMAND_USAGE_3"]);
	elseif ( arg1 == "show" ) then
		Broker_Volume.config.profile.minimap.hide = false;
		Broker_Volume:ShowHideMinimap();
	elseif ( arg1 == "master" and arg2 ) then
        local volume = tonumber(arg2);

        if ( Broker_Volume:IsInRange(volume) ) then
            volume = Broker_Volume:SetVolumeNum(volume);
            SetCVar("Sound_MasterVolume", volume, "Doh!");
            Broker_Volume:Message(L["MASTER_SET_TO"]..": "..floor(volume * 100).."%");
        end
    elseif ( arg1 == "effects" and arg2 ) then
        local volume = tonumber(arg2);

        if ( Broker_Volume:IsInRange(volume) ) then
            volume = Broker_Volume:SetVolumeNum(volume);
            SetCVar("Sound_SFXVolume", volume, "Doh!");
            Broker_Volume:Message(L["EFFECTS_SET_TO"]..": "..floor(volume * 100).."%");
        end
	elseif ( arg1 == "music" and arg2 ) then
        local volume = tonumber(arg2);

        if ( Broker_Volume:IsInRange(volume) ) then
            volume = Broker_Volume:SetVolumeNum(volume);
            SetCVar("Sound_MusicVolume", volume, "Doh!");
            Broker_Volume:Message(L["MUSIC_SET_TO"]..": "..floor(volume * 100).."%");
        end
	elseif ( arg1 == "ambience" and arg2 ) then
        local volume = tonumber(arg2);

        if ( Broker_Volume:IsInRange(volume) ) then
            volume = Broker_Volume:SetVolumeNum(volume);
            SetCVar("Sound_AmbienceVolume", volume, "Doh!");
            Broker_Volume:Message(L["AMBIENCE_SET_TO"]..": "..floor(volume * 100).."%");
        end
	else
		Broker_Volume:Message(L["COMMAND_USAGE_1"]);
		Broker_Volume:Message(L["COMMAND_USAGE_2"]);
		Broker_Volume:Message(L["COMMAND_USAGE_3"]);
	end
end

--- Round
-- @param num The number to round
-- @param idp Number of decimal
-- @return The rounded number
function Broker_Volume:Round(num, idp)
    local mult = 10 ^ ( idp or 0 );

    return math.floor(num * mult + 0.5) / mult;
end

--- Ceil decade
-- @param val The number to round
-- @return Rounded number
function Broker_Volume:CeilDecade(num)
    return ceil(num / 10) * 10;
end

--- Set a number to match volume cvar
-- @param val The number to set
-- @return The rounded number
function Broker_Volume:SetVolumeNum(num)
    num = Broker_Volume:CeilDecade(num);
    num = num / 100;
    num = Broker_Volume:Round(num, 1);

    return num;
end

--- Get the volume level in percent
-- @param cat Volume category
-- @return The volume as percent
function Broker_Volume:GetVolumePercent(cat)
    local volume = tonumber(GetCVar(cat));

    volume = floor(volume * 100);

    return volume;
end

--- Check if the number is between 0-100
-- @param num The number to test
function Broker_Volume:IsInRange(num)
    local inRange = false;

    if ( num >= 0 and num <= 100) then
        inRange = true;
    else
        Broker_Volume:Message(L["NOT_IN_RANGE"], true);
    end

    return inRange;
end

--- Blizzard default function Sound_ToggleMusic()
function Broker_Volume:ToggleMusic()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableMusic") == "0" ) then
            SetCVar("Sound_EnableMusic", 1, "Doh!");
            ActionStatus_DisplayMessage(MUSIC_ENABLED, true)
        else
            SetCVar("Sound_EnableMusic", 0, "Doh!");
            ActionStatus_DisplayMessage(MUSIC_DISABLED, true)
        end
    end
end

--- Blizzard default function Sound_ToggleSound()
function Broker_Volume:ToggleSound()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableSFX") == "0" ) then
            SetCVar("Sound_EnableSFX", 1, "Doh!");
            SetCVar("Sound_EnableAmbience", 1, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_ENABLED, true);
        else
            SetCVar("Sound_EnableSFX", 0, "Doh!");
            SetCVar("Sound_EnableAmbience", 0, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_DISABLED, true);
        end
    end
end

--- Toggle all soundss
function Broker_Volume:ToggleAll()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableSFX") == "0" ) then
            SetCVar("Sound_EnableSFX", 1, "Doh!");
            SetCVar("Sound_EnableAmbience", 1, "Doh!");
            SetCVar("Sound_EnableMusic", 1, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_ENABLED, true);
        else
            SetCVar("Sound_EnableSFX", 0, "Doh!");
            SetCVar("Sound_EnableAmbience", 0, "Doh!");
            SetCVar("Sound_EnableMusic", 0, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_DISABLED, true);
        end
    end
end

--- Show or hide the minimap icon
function Broker_Volume:ShowHideMinimap()
	if ( Broker_Volume.config.profile.minimap.hide ) then
		Broker_Volume:Message(L["HIDE_MINIMAP"], true);
		Broker_Volume.icon:Hide("Broker_VolumeObject");
	else
		Broker_Volume.icon:Show("Broker_VolumeObject");
	end
end

--- Update Broker
function Broker_Volume:UpdateBroker()
    if ( GetCVar("Sound_EnableAllSound") == "0" or GetCVar("Sound_EnableSFX") == "0" ) then
        Broker_Volume.ldb.text = L["MUTE"];
        Broker_Volume.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute";
    else
        local volume = Broker_Volume:GetVolumePercent("Sound_MasterVolume");

        Broker_Volume.ldb.text = volume.."%";
        if ( volume > 60 ) then
            Broker_Volume.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-max";
        elseif ( volume > 30 ) then
            Broker_Volume.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-medium";
        elseif ( volume > 0 ) then
            Broker_Volume.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-low";
        else
            Broker_Volume.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute";
        end
    end
end

--- Callback function for event VARIABLES_LOADED
function Broker_Volume:VARIABLES_LOADED()
    Broker_Volume:UpdateBroker();
end

--- Callback function for event CVAR_UPDATE
function Broker_Volume:CVAR_UPDATE()
    Broker_Volume:UpdateBroker();
end

--- Callback function for hook Sound_MasterVolumeUp()
function Broker_Volume:Sound_MasterVolumeUp()
    local volume = tonumber(GetCVar("Sound_MasterVolume"));

    volume = Broker_Volume:Round(volume, 1);
	if ( volume and volume <= 0.9) then
		SetCVar("Sound_MasterVolume", volume + 0.1, "Doh!");
	end
end

--- Callback function for hook Sound_MasterVolumeDown()
function Broker_Volume:Sound_MasterVolumeDown()
    local volume = tonumber(GetCVar("Sound_MasterVolume"));

    volume = Broker_Volume:Round(volume, 1);
	if ( volume and volume >= 0.1 ) then
		SetCVar("Sound_MasterVolume", volume - 0.1, "Doh!");
	end
end

--- Callback function for hook Sound_ToggleSound()
function Broker_Volume:Sound_ToggleSound()
    if ( Broker_Volume.config.profile.blizlike ) then
        Broker_Volume:ToggleMusic();
    else
        Broker_Volume:ToggleAll();
    end
end

--- Callback function for hook Sound_ToggleMusic()
function Broker_Volume:Sound_ToggleMusic()
    if ( Broker_Volume.config.profile.blizlike ) then
        Broker_Volume:ToggleMusic();
    else
        Broker_Volume:ToggleAll();
    end
end

-- ********************************************************************************
-- Configuration DB
-- ********************************************************************************

--- Default configuration table for AceDB
local defaults =
{
	profile =
	{
        blizlike = false,
		minimap =
		{
			hide = false
		}
	}
};

-- ********************************************************************************
-- Main
-- ********************************************************************************

--- AceAddon callback
-- Called after the addon is fully loaded
function Broker_Volume:OnInitialize()
    -- Config db
	Broker_Volume.config = LibStub("AceDB-3.0"):New("Broker_VolumeDB", defaults);

	-- LDB
	Broker_Volume.ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_VolumeObject",
	{
		type = "data source",
		text = "",
		label = L["ADDON_NAME"],
		icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute",
		tocname = "Broker_Volume",
		OnClick = function(self, button)
			if (button == "LeftButton") then
				if ( Broker_Volume.config.profile.blizlike ) then
                    Broker_Volume:ToggleMusic();
                else
                    Broker_Volume:ToggleAll();
                end
			elseif ( button == "RightButton" ) then
				if ( Broker_Volume.menuFrame.initialize ~= DropdownMenu ) then
					CloseDropDownMenus();
					Broker_Volume.menuFrame.initialize = DropdownMenu;
				end
				ToggleDropDownMenu(1, nil, Broker_Volume.menuFrame, self:GetName(), 0, 0);
				GameTooltip:Hide();
			end
		end,
		OnTooltipShow = function(tooltip)
			tooltip:AddDoubleLine(Broker_Volume.color["WHITE"]..L["ADDON_NAME"], Broker_Volume.color["GREEN"].." v"..Broker_Volume.version);
			tooltip:AddLine(" ");

			tooltip:AddLine(L["MASTER_VOLUME"]..": "..Broker_Volume:GetVolumePercent("Sound_MasterVolume").."%");
            tooltip:AddLine(L["EFFECTS_VOLUME"]..": "..Broker_Volume:GetVolumePercent("Sound_SFXVolume").."%");
            tooltip:AddLine(L["MUSIC_VOLUME"]..": "..Broker_Volume:GetVolumePercent("Sound_MusicVolume").."%");
            tooltip:AddLine(L["AMBIENCE_VOLUME"]..": "..Broker_Volume:GetVolumePercent("Sound_AmbienceVolume").."%");

			tooltip:AddLine(" ");
			tooltip:AddLine(string.format(L["TOOLTIP_TIPS"], Broker_Volume.color["WARRIOR"], Broker_Volume.color["GREEN"], Broker_Volume.color["WARRIOR"], Broker_Volume.color["GREEN"]));
		end,
        OnMouseWheel = function(delta)
            print(tostring(delta));
        end
	});

	-- LDBIcon
	Broker_Volume.icon:Register("Broker_VolumeObject", Broker_Volume.ldb, Broker_Volume.config.profile.minimap);

    -- Menu frame & table
	Broker_Volume.menuFrame = CreateFrame("Frame", "Broker_VolumeMenuFrame");
	Broker_Volume.menuFrame.displayMode = "MENU";
	Broker_Volume.menuFrame.info = {};

	-- Events
	Broker_Volume:RegisterEvent("VARIABLES_LOADED");
	Broker_Volume:RegisterEvent("CVAR_UPDATE");

    -- Hooks
    Broker_Volume:RawHook("Sound_MasterVolumeUp", true);
    Broker_Volume:RawHook("Sound_MasterVolumeDown", true);
    Broker_Volume:RawHook("Sound_ToggleSound", true);
    Broker_Volume:RawHook("Sound_ToggleMusic", true);

    -- Commands
    Broker_Volume:RegisterChatCommand("brokervolume", "SlashCommand");
	Broker_Volume:RegisterChatCommand("bv", "SlashCommand");
end
